import contains from "dom-helpers/contains";
import listen from "dom-helpers/listen";
import ownerDocument from "dom-helpers/ownerDocument";
import { createEffect, createSignal, mergeProps, onCleanup } from "solid-js";
const escapeKeyCode = 27;
const noop = () => { };
function isLeftClickEvent(event) {
    return event.button === 0;
}
function isModifiedEvent(event) {
    return !!(event.metaKey || event.altKey || event.ctrlKey || event.shiftKey);
}
/**
 * The `useRootClose` hook registers your callback on the document
 * when rendered. Powers the `<Overlay/>` component. This is used achieve modal
 * style behavior where your callback is triggered when the user tries to
 * interact with the rest of the document or hits the `esc` key.
 *
 * @param {Ref<HTMLElement>| HTMLElement} ref  The element boundary
 * @param {function} onRootClose
 * @param {object=}  options
 * @param {boolean=} options.disabled
 * @param {string=}  options.clickTrigger The DOM event name (click, mousedown, etc) to attach listeners on
 */
function useRootClose(ref, onRootClose, rootCloseoptions = {}) {
    let options = mergeProps({ clickTrigger: "click" }, rootCloseoptions);
    const [preventMouseRootCloseRef, setPreventMouseRootCloseRef] = createSignal(false);
    const onClose = onRootClose || noop;
    const handleMouseCapture = (e) => {
        const currentTarget = ref();
        setPreventMouseRootCloseRef(!currentTarget ||
            isModifiedEvent(e) ||
            !isLeftClickEvent(e) ||
            !!contains(currentTarget, e.target));
    };
    const handleMouse = (e) => {
        if (!preventMouseRootCloseRef()) {
            onClose(e);
        }
    };
    const handleKeyUp = (e) => {
        if (e.keyCode === escapeKeyCode) {
            onClose(e);
        }
    };
    createEffect(() => {
        if (options.disabled || ref() == null)
            return;
        // Store the current event to avoid triggering handlers immediately
        // https://github.com/facebook/react/issues/20074
        let currentEvent = window.event;
        const doc = ownerDocument(ref());
        // Use capture for this listener so it fires before React's listener, to
        // avoid false positives in the contains() check below if the target DOM
        // element is removed in the React mouse callback.
        const removeMouseCaptureListener = listen(doc, options.clickTrigger, handleMouseCapture, true);
        const removeMouseListener = listen(doc, options.clickTrigger, (e) => {
            // skip if this event is the same as the one running when we added the handlers
            if (e === currentEvent) {
                currentEvent = undefined;
                return;
            }
            handleMouse(e);
        });
        const removeKeyupListener = listen(doc, "keyup", (e) => {
            // skip if this event is the same as the one running when we added the handlers
            if (e === currentEvent) {
                currentEvent = undefined;
                return;
            }
            handleKeyUp(e);
        });
        let mobileSafariHackListeners = [];
        if ("ontouchstart" in doc.documentElement) {
            mobileSafariHackListeners = [].slice
                .call(doc.body.children)
                .map((el) => listen(el, "mousemove", noop));
        }
        onCleanup(() => {
            removeMouseCaptureListener();
            removeMouseListener();
            removeKeyupListener();
            mobileSafariHackListeners.forEach((remove) => remove());
        });
    });
}
export default useRootClose;
